#include "LSM6DS3.h"
#include "Wire.h"

#define CLEAR_STEP      true
#define NOT_CLEAR_STEP  false

// Haptic motor pin
#define HAPTIC_PIN      10

// Cadence settings (steps per minute - both feet combined)
#define OPTIMAL_CADENCE_MIN   170
#define OPTIMAL_CADENCE_MAX   180
#define REFERENCE_CADENCE_SLOW  170  // Target cadence when too slow
#define REFERENCE_CADENCE_FAST  180  // Target cadence when too fast
// Note: Since haptic is on one foot only, we'll vibrate at half these rates

// Vibration settings
#define PULSE_DURATION    50    // Short uniform pulse (milliseconds)

// Create instance of LSM6DS3
LSM6DS3 pedometer(I2C_MODE, 0x6A);

// Variables for cadence calculation
unsigned long lastStepTime = 0;
unsigned long currentStepTime = 0;
uint16_t lastStepCount = 0;
uint16_t currentStepCount = 0;
float currentCadence = 0;

// Reference cadence timing
unsigned long lastReferenceVibration = 0;
unsigned long referenceInterval = 0;
bool isProvidingReference = false;

void setup() {
    Serial.begin(9600);
    while (!Serial);
    
    // Initialize haptic motor pin
    pinMode(HAPTIC_PIN, OUTPUT);
    digitalWrite(HAPTIC_PIN, LOW);
    
    // Initialize pedometer
    if (pedometer.begin() != 0) {
        Serial.println("Device error");
    } else {
        Serial.println("Device OK!");
    }

    // Configure LSM6DS3 as pedometer
    if (0 != config_pedometer(NOT_CLEAR_STEP)) {
        Serial.println("Configure pedometer fail!");
    }
    Serial.println("Success to Configure pedometer!");
    
    // Welcome vibration pattern
    welcomeVibration();
    
    // Initialize step count
    lastStepCount = readStepCount();
}

void loop() {
    // Read current step count
    currentStepCount = readStepCount();
    
    // Check if new step detected
    if (currentStepCount > lastStepCount) {
        currentStepTime = millis();
        
        // Update cadence calculation
        updateCadence();
        
        // Display information
        displayInfo(currentStepCount);
        
        // Update last step count
        lastStepCount = currentStepCount;
    }
    
    // Handle reference cadence vibrations
    handleReferenceCadence();
    
    // Small delay to prevent excessive polling
    delay(20);
}

// Read step count from sensor
uint16_t readStepCount() {
    uint8_t dataByte = 0;
    uint16_t steps = 0;
    
    pedometer.readRegister(&dataByte, LSM6DS3_ACC_GYRO_STEP_COUNTER_H);
    steps = (dataByte << 8) & 0xFFFF;
    
    pedometer.readRegister(&dataByte, LSM6DS3_ACC_GYRO_STEP_COUNTER_L);
    steps |= dataByte;
    
    return steps;
}

// Update cadence calculation
void updateCadence() {
    if (lastStepTime > 0 && currentStepTime > lastStepTime) {
        // Calculate time between step detections
        unsigned long timeBetweenDetections = currentStepTime - lastStepTime;
        uint16_t stepsDiff = currentStepCount - lastStepCount;
        
        if (stepsDiff > 0 && timeBetweenDetections > 0) {
            // Calculate cadence based on steps detected
            float instantCadence = (stepsDiff * 60000.0) / timeBetweenDetections;
            
            // Smooth the cadence reading
            if (currentCadence == 0) {
                currentCadence = instantCadence;
            } else {
                currentCadence = (currentCadence * 0.7) + (instantCadence * 0.3); // Weighted average
            }
            
            // Update reference cadence settings based on current cadence
            updateReferenceSettings();
        }
    }
    
    lastStepTime = currentStepTime;
}

// Update reference cadence settings
void updateReferenceSettings() {
    if (currentCadence >= OPTIMAL_CADENCE_MIN && currentCadence <= OPTIMAL_CADENCE_MAX) {
        // Optimal cadence - no feedback needed
        isProvidingReference = false;
    } else if (currentCadence < OPTIMAL_CADENCE_MIN) {
        // Too slow - provide 170 SPM reference (85 per foot)
        isProvidingReference = true;
        referenceInterval = 60000 / (REFERENCE_CADENCE_SLOW / 2); // Divide by 2 for single foot
    } else if (currentCadence > OPTIMAL_CADENCE_MAX) {
        // Too fast - provide 180 SPM reference (90 per foot)
        isProvidingReference = true;
        referenceInterval = 60000 / (REFERENCE_CADENCE_FAST / 2); // Divide by 2 for single foot
    }
}

// Handle reference cadence vibrations
void handleReferenceCadence() {
    if (isProvidingReference && referenceInterval > 0) {
        unsigned long currentTime = millis();
        
        // Check if it's time for the next reference vibration
        if (currentTime - lastReferenceVibration >= referenceInterval) {
            vibrate();
            lastReferenceVibration = currentTime;
        }
    }
}

// Vibrate the haptic motor with uniform pulse
void vibrate() {
    digitalWrite(HAPTIC_PIN, HIGH);
    delay(PULSE_DURATION);
    digitalWrite(HAPTIC_PIN, LOW);
}

// Welcome vibration pattern to indicate system ready
void welcomeVibration() {
    Serial.println("System ready - Welcome vibration!");
    for (int i = 0; i < 3; i++) {
        vibrate();
        delay(200);
    }
}

// Display current information
void displayInfo(uint16_t steps) {
    Serial.print("Steps: ");
    Serial.print(steps);
    Serial.print(" | Cadence: ");
    
    if (currentCadence > 0) {
        Serial.print(currentCadence, 1);
        Serial.print(" SPM");
        
        if (currentCadence < OPTIMAL_CADENCE_MIN) {
            Serial.print(" [TOO SLOW - Match left foot to vibrations]");
        } else if (currentCadence > OPTIMAL_CADENCE_MAX) {
            Serial.print(" [TOO FAST - Match left foot to vibrations]");
        } else {
            Serial.print(" [OPTIMAL - No feedback]");
        }
        
        if (isProvidingReference) {
            Serial.print(" | Vibrating at: ");
            Serial.print(60000 / referenceInterval / 2); // Show per-foot rate
            Serial.print(" beats/min (");
            Serial.print(60000 / referenceInterval);
            Serial.print(" SPM total)");
        }
    } else {
        Serial.print("Calculating...");
    }
    
    Serial.println();
}

// Setup pedometer mode
int config_pedometer(bool clearStep) {
    uint8_t errorAccumulator = 0;
    uint8_t dataToWrite = 0;

    // Setup the accelerometer
    dataToWrite = 0;
    dataToWrite |= LSM6DS3_ACC_GYRO_FS_XL_2g;
    dataToWrite |= LSM6DS3_ACC_GYRO_ODR_XL_26Hz;

    // Step 1: Configure ODR-26Hz and FS-2g
    errorAccumulator += pedometer.writeRegister(LSM6DS3_ACC_GYRO_CTRL1_XL, dataToWrite);

    // Step 2: Set bit Zen_G, Yen_G, Xen_G, FUNC_EN, PEDO_RST_STEP(1 or 0)
    if (clearStep) {
        errorAccumulator += pedometer.writeRegister(LSM6DS3_ACC_GYRO_CTRL10_C, 0x3E);
    } else {
        errorAccumulator += pedometer.writeRegister(LSM6DS3_ACC_GYRO_CTRL10_C, 0x3C);
    }

    // Step 3: Enable pedometer algorithm
    errorAccumulator += pedometer.writeRegister(LSM6DS3_ACC_GYRO_TAP_CFG1, 0x40);

    // Step 4: Step Detector interrupt driven to INT1 pin (even though we're not using the physical pin)
    errorAccumulator += pedometer.writeRegister(LSM6DS3_ACC_GYRO_INT1_CTRL, 0x10);

    return errorAccumulator;
}